@@ -14,14 +14,16 @@
 // You should have received a copy of the GNU General Public License
 // along with the snarkOS library. If not, see <https://www.gnu.org/licenses/>.
 
-use crate::{master::SyncInbound, sync::master::SyncMaster, *};
-use snarkos_metrics::{self as metrics, inbound, misc};
+use crate::*;
+use snarkos_metrics::{self as metrics, misc};
 
 use anyhow::*;
 use chrono::{DateTime, Utc};
 use once_cell::sync::OnceCell;
 use rand::{thread_rng, Rng};
 use snarkos_storage::DynStorage;
+#[cfg(not(feature = "test"))]
+use std::time::Duration;
 use std::{
     net::SocketAddr,
     ops::Deref,
@@ -55,13 +57,11 @@ pub struct InnerNode {
     /// The current state of the node.
     state: StateCode,
     /// The local address of this node.
-    pub local_address: OnceCell<SocketAddr>,
+    pub local_addr: OnceCell<SocketAddr>,
     /// The pre-configured parameters of this node.
     pub config: Config,
-    /// The inbound handler of this node.
-    pub inbound: Inbound,
     /// The cache of node's inbound messages.
-    pub inbound_cache: Mutex<Cache>,
+    pub inbound_cache: Mutex<BlockCache<{ crate::BLOCK_CACHE_SIZE }>>,
     /// The list of connected and disconnected peers of this node.
     pub peer_book: PeerBook,
     /// Tracks the known network crawled by this node.
@@ -120,12 +120,11 @@ impl Node {
     /// Creates a new instance of `Node`.
     pub async fn new(config: Config, storage: DynStorage) -> Result<Self, NetworkError> {
         let node = Self(Arc::new(InnerNode {
-            id: thread_rng().gen(),
+            id: config.node_id.unwrap_or_else(|| thread_rng().gen()),
             state: Default::default(),
-            local_address: Default::default(),
+            local_addr: Default::default(),
             config,
             storage,
-            inbound: Default::default(),
             inbound_cache: Default::default(),
             peer_book: PeerBook::spawn(),
             sync: Default::default(),
@@ -139,6 +138,9 @@ impl Node {
         }));
 
         if node.config.is_crawler() {
+            // Safe since crawlers don't start the listener service.
+            node.set_local_addr(node.config.desired_address);
+
             // Safe since this can only ever be set here.
             node.known_network.set(KnownNetwork::default()).unwrap();
         }
@@ -175,20 +177,6 @@ impl Node {
     }
 
     pub async fn start_services(&self) {
-        let node_clone = self.clone();
-        let mut receiver = self.inbound.take_receiver().await;
-        let incoming_task = task::spawn(async move {
-            loop {
-                if let Err(e) = node_clone.process_incoming_messages(&mut receiver).await {
-                    metrics::increment_counter!(inbound::ALL_FAILURES);
-                    error!("Node error: {}", e);
-                } else {
-                    metrics::increment_counter!(inbound::ALL_SUCCESSES);
-                }
-            }
-        });
-        self.register_task(incoming_task);
-
         let node_clone: Node = self.clone();
         let peer_sync_interval = self.config.peer_sync_interval();
         let peering_task = task::spawn(async move {
@@ -217,25 +205,60 @@ impl Node {
             self.register_task(known_network_task);
         }
 
-        let node_clone = self.clone();
-        let state_tracking_task = task::spawn(async move {
-            loop {
-                sleep(std::time::Duration::from_secs(5)).await;
+        if !self.config.is_crawler() {
+            let node_clone = self.clone();
+            let state_tracking_task = task::spawn(async move {
+                loop {
+                    sleep(std::time::Duration::from_secs(5)).await;
 
-                // Report node's current state.
-                trace!("Node state: {:?}", node_clone.state());
-            }
-        });
-        self.register_task(state_tracking_task);
+                    // Report node's current state.
+                    trace!("Node state: {:?}", node_clone.state());
+                }
+            });
+            self.register_task(state_tracking_task);
+        }
 
         if self.sync().is_some() {
-            let bootnodes = self.config.bootnodes();
+            let node_clone = self.clone();
+            let block_sync_interval = node_clone.expect_sync().block_sync_interval();
+            let sync_block_task = task::spawn(async move {
+                loop {
+                    if node_clone.peer_book.get_connected_peer_count() != 0 {
+                        let is_syncing_blocks = node_clone.is_syncing_blocks();
 
+                        if !is_syncing_blocks {
+                            if let Err(e) = node_clone.run_sync().await {
+                                error!("failed sync process: {:?}", e);
+                            }
+                            node_clone.finished_syncing_blocks();
+                        }
+                    }
+
+                    sleep(block_sync_interval).await;
+                }
+            });
+            self.register_task(sync_block_task);
+
+            // An arbitrary short delay before mempool sync attempts begin, to ensure that block sync
+            // has been attempted first.
+            #[cfg(not(feature = "test"))]
+            sleep(Duration::from_secs(5)).await;
+
+            let bootnodes = self.config.bootnodes();
             let node_clone = self.clone();
             let mempool_sync_interval = node_clone.expect_sync().mempool_sync_interval();
             let sync_mempool_task = task::spawn(async move {
                 loop {
                     if !node_clone.is_syncing_blocks() {
+                        // Ensure that it's not just a short "break" from syncing blocks
+                        #[cfg(not(feature = "test"))]
+                        if let Some(elapsed) = node_clone.expect_sync().time_since_last_block_sync() {
+                            if elapsed < Duration::from_secs(60) {
+                                sleep(mempool_sync_interval).await;
+                                continue;
+                            }
+                        }
+
                         // TODO (howardwu): Add some random sync nodes beyond this approach
                         //  to ensure some diversity in mempool state that is fetched.
                         //  For now, this is acceptable because we propogate the mempool to
@@ -270,25 +293,6 @@ impl Node {
                 }
             });
             self.register_task(sync_mempool_task);
-
-            let node_clone = self.clone();
-            let block_sync_interval = node_clone.expect_sync().block_sync_interval();
-            let sync_block_task = task::spawn(async move {
-                loop {
-                    let is_syncing_blocks = node_clone.is_syncing_blocks();
-
-                    if !is_syncing_blocks {
-                        node_clone.register_block_sync_attempt();
-                        if let Err(e) = node_clone.run_sync().await {
-                            error!("failed sync process: {:?}", e);
-                        }
-                        node_clone.finished_syncing_blocks();
-                    }
-
-                    sleep(block_sync_interval).await;
-                }
-            });
-            self.register_task(sync_block_task);
         }
     }
 
@@ -312,9 +316,17 @@ impl Node {
         self.threads.append(handle);
     }
 
+    /// Sets the local address of the node to the given value.
     #[inline]
-    pub fn local_address(&self) -> Option<SocketAddr> {
-        self.local_address.get().copied()
+    pub fn set_local_addr(&self, addr: SocketAddr) {
+        self.local_addr
+            .set(addr)
+            .expect("local address was set more than once!");
+    }
+
+    #[inline]
+    pub fn expect_local_addr(&self) -> SocketAddr {
+        self.local_addr.get().copied().expect("no address set!")
     }
 
     #[inline]
@@ -322,14 +334,6 @@ impl Node {
         self.shutting_down.load(Ordering::Relaxed)
     }
 
-    /// Sets the local address of the node to the given value.
-    #[inline]
-    pub fn set_local_address(&self, addr: SocketAddr) {
-        self.local_address
-            .set(addr)
-            .expect("local address was set more than once!");
-    }
-
     pub async fn initialize_metrics(&self) -> Result<()> {
         debug!("Initializing metrics");
         if let Some(metrics_task) = snarkos_metrics::initialize() {
@@ -337,20 +341,19 @@ impl Node {
         }
 
         // The node can already be at some non-zero height.
-        metrics::gauge!(misc::BLOCK_HEIGHT, self.storage.canon().await?.block_height as f64);
+        if self.sync().is_some() {
+            metrics::gauge!(misc::BLOCK_HEIGHT, self.storage.canon().await?.block_height as f64);
+        }
+
         Ok(())
     }
 
     pub fn version(&self) -> Version {
-        Version::new(
-            crate::PROTOCOL_VERSION,
-            self.local_address().map(|x| x.port()).unwrap_or_default(),
-            self.id,
-        )
+        Version::new(crate::PROTOCOL_VERSION, self.expect_local_addr().port(), self.id)
     }
 
     pub async fn run_sync(&self) -> Result<()> {
-        let (master, sender) = SyncMaster::new(self.clone());
+        let (master, sender) = SyncAggro::new(self.clone());
         *self.master_dispatch.write().await = Some(sender);
         master.run().await
     }
