@@ -19,8 +19,7 @@ use std::{
     time::Duration,
 };
 
-use futures::{select, FutureExt};
-use tokio::{net::TcpStream, sync::mpsc};
+use tokio::{net::TcpStream, sync::mpsc, time::timeout};
 
 use snarkos_metrics::{self as metrics, connections::*};
 
@@ -37,6 +36,14 @@ impl Peer {
             self.set_connecting();
             match self.inner_connect(node.version()).await {
                 Err(e) => {
+                    event_target
+                        .send(PeerEvent {
+                            address: self.address,
+                            data: PeerEventData::FailHandshake,
+                        })
+                        .await
+                        .ok();
+                    metrics::increment_gauge!(snarkos_metrics::queues::PEER_EVENTS, 1.0);
                     self.fail();
                     if !e.is_trivial() {
                         error!(
@@ -77,6 +84,8 @@ impl Peer {
                         })
                         .await
                         .ok();
+                    metrics::increment_gauge!(snarkos_metrics::queues::PEER_EVENTS, 1.0);
+
                     if let Err(e) = self.run(node, network, receiver).await {
                         if !e.is_trivial() {
                             self.fail();
@@ -94,15 +103,15 @@ impl Peer {
                     metrics::decrement_gauge!(CONNECTED, 1.0);
                 }
             }
-            let state = self.status;
             self.set_disconnected();
             event_target
                 .send(PeerEvent {
                     address: self.address,
-                    data: PeerEventData::Disconnect(self, state),
+                    data: PeerEventData::Disconnect(Box::new(self)),
                 })
                 .await
                 .ok();
+            metrics::increment_gauge!(snarkos_metrics::queues::PEER_EVENTS, 1.0);
         });
     }
 
@@ -110,15 +119,17 @@ impl Peer {
         metrics::increment_gauge!(CONNECTING, 1.0);
         let _x = defer::defer(|| metrics::decrement_gauge!(CONNECTING, 1.0));
 
-        let tcp_stream;
-        select! {
-            stream = TcpStream::connect(self.address).fuse() => {
-                tcp_stream = stream?;
-            },
-            _ = tokio::time::sleep(Duration::from_secs(CONNECTION_TIMEOUT_SECS)).fuse() => {
-                return Err(NetworkError::Io(IoError::new(ErrorKind::TimedOut, "connection timed out")));
-            },
+        match timeout(
+            Duration::from_secs(CONNECTION_TIMEOUT_SECS),
+            TcpStream::connect(self.address),
+        )
+        .await
+        {
+            Ok(stream) => self.inner_handshake_initiator(stream?, our_version).await,
+            Err(_) => Err(NetworkError::Io(IoError::new(
+                ErrorKind::TimedOut,
+                "connection timed out",
+            ))),
         }
-        self.inner_handshake_initiator(tcp_stream, our_version).await
     }
 }
