@@ -20,7 +20,7 @@ use tokio::{net::TcpStream, sync::mpsc};
 
 use snarkos_metrics::{self as metrics, connections::*};
 
-use crate::{NetworkError, Node, Peer, PeerEvent, PeerEventData, PeerHandle, PeerStatus, Version};
+use crate::{NetworkError, Node, Peer, PeerEvent, PeerEventData, PeerHandle, Version};
 
 use super::{network::PeerIOHandle, PeerAction};
 
@@ -41,6 +41,7 @@ impl Peer {
                         })
                         .await
                         .ok();
+                    metrics::increment_gauge!(snarkos_metrics::queues::PEER_EVENTS, 1.0);
                     return;
                 }
                 Ok(x) => x,
@@ -58,6 +59,8 @@ impl Peer {
                 })
                 .await
                 .ok();
+            metrics::increment_gauge!(snarkos_metrics::queues::PEER_EVENTS, 1.0);
+
             if let Err(e) = peer.run(node, network, receiver).await {
                 if !e.is_trivial() {
                     peer.fail();
@@ -77,10 +80,11 @@ impl Peer {
             event_target
                 .send(PeerEvent {
                     address: peer.address,
-                    data: PeerEventData::Disconnect(peer, PeerStatus::Connected),
+                    data: PeerEventData::Disconnect(Box::new(peer)),
                 })
                 .await
                 .ok();
+            metrics::increment_gauge!(snarkos_metrics::queues::PEER_EVENTS, 1.0);
         });
     }
 
