@@ -15,26 +15,27 @@
 // along with the snarkOS library. If not, see <https://www.gnu.org/licenses/>.
 
 use std::{
+    future::Future,
     net::SocketAddr,
     sync::{
         atomic::{AtomicU32, Ordering},
         Arc,
     },
+    time::Instant,
 };
 
-use futures::Future;
 use mpmc_map::MpmcMap;
-use rand::prelude::IteratorRandom;
+use rand::{prelude::IteratorRandom, rngs::SmallRng, SeedableRng};
 use tokio::{net::TcpStream, sync::mpsc};
 
 use snarkos_metrics::{self as metrics, connections::*};
 
-use crate::{NetworkError, Node, Payload, Peer, PeerEvent, PeerEventData, PeerHandle, PeerStatus};
+use crate::{NetworkError, Node, Payload, Peer, PeerEvent, PeerEventData, PeerHandle};
 
 ///
 /// A data structure for storing the history of all peers with this node server.
 ///
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct PeerBook {
     disconnected_peers: MpmcMap<SocketAddr, Peer>,
     connected_peers: MpmcMap<SocketAddr, PeerHandle>,
@@ -53,6 +54,8 @@ impl PeerBookRef {
     // gets terminated when sender is dropped from PeerBook
     async fn handle_peer_events(self, mut receiver: mpsc::Receiver<PeerEvent>) {
         while let Some(event) = receiver.recv().await {
+            metrics::decrement_gauge!(snarkos_metrics::queues::PEER_EVENTS, 1.0);
+
             match event.data {
                 PeerEventData::Connected(handle) => {
                     self.pending_connections.fetch_sub(1, Ordering::SeqCst);
@@ -61,18 +64,17 @@ impl PeerBookRef {
                         old_peer.disconnect().await;
                     }
                 }
-                PeerEventData::Disconnect(peer, status) => {
+                PeerEventData::Disconnect(peer) => {
                     self.connected_peers.remove(peer.address).await;
 
+                    let queued_inbound_message_count = peer.queued_inbound_message_count.swap(0, Ordering::SeqCst);
+                    metrics::decrement_gauge!(snarkos_metrics::queues::INBOUND, queued_inbound_message_count as f64);
                     let queued_outbound_message_count = peer.queued_outbound_message_count.swap(0, Ordering::SeqCst);
                     metrics::decrement_gauge!(snarkos_metrics::queues::OUTBOUND, queued_outbound_message_count as f64);
 
-                    if self.disconnected_peers.insert(peer.address, peer).await.is_none() {
+                    if self.disconnected_peers.insert(peer.address, *peer).await.is_none() {
                         metrics::increment_gauge!(DISCONNECTED, 1.0);
                     }
-                    if status == PeerStatus::Connecting {
-                        self.pending_connections.fetch_sub(1, Ordering::SeqCst);
-                    }
                 }
                 PeerEventData::FailHandshake => {
                     self.pending_connections.fetch_sub(1, Ordering::SeqCst);
@@ -211,14 +213,17 @@ impl PeerBook {
         self.for_each_peer(move |peer| {
             let payload = payload.clone();
             async move {
-                peer.send_payload(payload).await;
+                peer.send_payload(payload, None).await;
             }
         })
         .await;
     }
 
-    pub async fn send_to(&self, address: SocketAddr, payload: Payload) -> Option<()> {
-        self.connected_peers.get(&address)?.send_payload(payload).await;
+    pub async fn send_to(&self, address: SocketAddr, payload: Payload, time_received: Option<Instant>) -> Option<()> {
+        self.connected_peers
+            .get(&address)?
+            .send_payload(payload, time_received)
+            .await;
         Some(())
     }
 
@@ -283,7 +288,10 @@ impl PeerBook {
             .collect::<Vec<Peer>>();
         let count_total_higher = peers.len();
 
-        Some((peers.into_iter().choose(&mut rand::thread_rng())?, count_total_higher))
+        Some((
+            peers.into_iter().choose(&mut SmallRng::from_entropy())?,
+            count_total_higher,
+        ))
     }
 
     /// Cancels any expected sync block counts from all peers.
