@@ -14,9 +14,13 @@
 // You should have received a copy of the GNU General Public License
 // along with the snarkOS library. If not, see <https://www.gnu.org/licenses/>.
 
-use std::{cmp, net::SocketAddr, time::Duration};
+use std::{
+    cmp,
+    net::SocketAddr,
+    time::{Duration, Instant},
+};
 
-use rand::{prelude::SliceRandom, seq::IteratorRandom};
+use rand::{prelude::SliceRandom, rngs::SmallRng, seq::IteratorRandom, SeedableRng};
 use tokio::task;
 
 use snarkos_metrics::{self as metrics, connections::*};
@@ -115,7 +119,7 @@ impl Node {
             let random_bootnodes = self
                 .config
                 .bootnodes()
-                .choose_multiple(&mut rand::thread_rng(), 2)
+                .choose_multiple(&mut SmallRng::from_entropy(), 2)
                 .copied()
                 .collect::<Vec<_>>();
 
@@ -142,7 +146,7 @@ impl Node {
         debug!("Connecting to {}...", remote_address);
 
         // Local address must be known by now.
-        let own_address = self.local_address().unwrap();
+        let own_address = self.expect_local_addr();
 
         // Don't connect if maximum number of connections has been reached.
         if !self.can_connect() {
@@ -174,7 +178,7 @@ impl Node {
     ///
     pub async fn connect_to_addresses(&self, addrs: &[SocketAddr]) {
         // Local address must be known by now.
-        let own_address = self.local_address().unwrap();
+        let own_address = self.expect_local_addr();
 
         for node_addr in addrs
             .iter()
@@ -206,7 +210,7 @@ impl Node {
     ///
     async fn connect_to_disconnected_peers(&self, count: usize) {
         // Local address must be known by now.
-        let own_address = self.local_address().unwrap();
+        let own_address = self.expect_local_addr();
 
         // If this node is not a bootnode, attempt to satisfy the minimum number of peer connections.
         let random_peers = {
@@ -232,7 +236,7 @@ impl Node {
             if !self.config.is_regular_node() {
                 addr_iter.take(count).collect()
             } else {
-                addr_iter.choose_multiple(&mut rand::thread_rng(), count)
+                addr_iter.choose_multiple(&mut SmallRng::from_entropy(), count)
             }
         };
 
@@ -282,7 +286,11 @@ impl Node {
         trace!("Broadcasting `Ping` messages");
 
         // Consider peering tests that don't use the sync layer.
-        let current_block_height = self.storage.canon().await?.block_height as u32;
+        let current_block_height = if self.sync().is_some() {
+            self.storage.canon().await?.block_height as u32
+        } else {
+            0
+        };
 
         self.peer_book.broadcast(Payload::Ping(current_block_height)).await;
         Ok(())
@@ -301,7 +309,7 @@ impl Node {
         }
     }
 
-    pub(crate) async fn send_peers(&self, remote_address: SocketAddr) {
+    pub(crate) async fn send_peers(&self, remote_address: SocketAddr, time_received: Option<Instant>) {
         // Broadcast the sanitized list of connected peers back to the requesting peer.
 
         use crate::Peer;
@@ -345,20 +353,22 @@ impl Node {
 
         // Limit set size.
         let peers = peers
-            .choose_multiple(&mut rand::thread_rng(), crate::SHARED_PEER_COUNT)
+            .choose_multiple(&mut SmallRng::from_entropy(), crate::SHARED_PEER_COUNT)
             .copied()
             .collect();
 
-        self.peer_book.send_to(remote_address, Payload::Peers(peers)).await;
+        self.peer_book
+            .send_to(remote_address, Payload::Peers(peers), time_received)
+            .await;
     }
 
     /// A node has sent their list of peer addresses.
     /// Add all new/updated addresses to our disconnected.
     /// The connection handler will be responsible for sending out handshake requests to them.
     pub(crate) async fn process_inbound_peers(&self, source: SocketAddr, peers: Vec<SocketAddr>) {
-        let local_address = self.local_address().unwrap(); // the address must be known by now
+        let local_addr = self.expect_local_addr(); // the address must be known by now
 
-        for peer_address in peers.iter().filter(|&peer_addr| *peer_addr != local_address) {
+        for peer_address in peers.iter().filter(|&peer_addr| *peer_addr != local_addr) {
             // Inform the peer book that we found a peer.
             // The peer book will determine if we have seen the peer before,
             // and include the peer if it is new.
@@ -380,7 +390,7 @@ impl Node {
         let max_peers = self.config.maximum_number_of_connected_peers() as usize;
 
         if num_connected > max_peers {
-            warn!(
+            debug!(
                 "Max number of connections ({} connected; max: {}) reached",
                 num_connected, max_peers
             );
